package cologne.eck.all_peas.gui.menu;

/*
 * PeaFactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */
import java.awt.Font;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ResourceBundle;

import javax.accessibility.AccessibleContext;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;

import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.Attachments;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.gui.PeaDialog;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.all_peas.vm_specific.JREProperties;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.peafactory.crypto.HashStuff;
import cologne.eck.peafactory.crypto.kdf.KeyDerivation;
import cologne.eck.tools.Comparator;
import cologne.eck.tools.Converter;
import cologne.eck.tools.ExceptionHelper;
import cologne.eck.tools.TestLog;
import cologne.eck.tools.UnexpectedValueException;
import cologne.eck.tools.WebTools;
import settings.PeaSettings;



@SuppressWarnings("serial")
public class HelpMenu extends JMenu implements ActionListener {
	
	private ResourceBundle languagesBundle;
	
	private Window owner;// lock frame or password dialog

	/**
	 * Help menu for menu bar
	 * 
	 * @param _owner	the root window (PswDialogView or lockFrame)
	 */
	public HelpMenu(Window _owner) {
		
		this.owner = _owner;
		Font font = new Font(Font.SANS_SERIF, Font.PLAIN, PeaProperties.getFontSize());
		this.setFont(font);
		languagesBundle = (ResourceBundle) JREProperties.getBundle();
		this.setText(languagesBundle.getString("help"));
		this.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a menu to get help and further informations.");
//		int fontSize = PswDialogView.getFontsize();
//		this.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, fontSize));
		//help.setMnemonic(KeyEvent.VK_H);
		this.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("help_mnemonic")).getKeyCode());
		
		JMenuItem instructionItem = new JMenuItem(languagesBundle.getString("instruction"));
		instructionItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component displays a short instruction text.");
		instructionItem.setActionCommand("instruction");
		instructionItem.setFont(font);
		//instructionItem.setMnemonic(KeyEvent.VK_I);
		instructionItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("instruction_mnemonic")).getKeyCode());
		instructionItem.addActionListener(this);
		this.add(instructionItem);
		
		JMenuItem webItem = new JMenuItem(languagesBundle.getString("web"));
		webItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component displays addresses to find further informations in the web.");
		webItem.setActionCommand("web");
		webItem.setFont(font);
		//webItem.setMnemonic(KeyEvent.VK_I);
		webItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("web_mnemonic")).getKeyCode());
		webItem.addActionListener(this);
		this.add(webItem);
		
		if (_owner instanceof PswDialogView) {
			JMenuItem rememberFileItem = new JMenuItem(languagesBundle.getString("edit_remembered_files"));
			rememberFileItem.getAccessibleContext().
			setAccessibleDescription(
					"Clicking this component opens a dialog to help editing remembered files.");
			rememberFileItem.setActionCommand("remembered-files");
			rememberFileItem.setFont(font);
			//webItem.setMnemonic(KeyEvent.VK_I);
			//pathFileItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("path_file_mnemonic")).getKeyCode());
			rememberFileItem.addActionListener(this);
			this.add(rememberFileItem);				
		}
		
		JMenuItem pathFileItem = new JMenuItem(languagesBundle.getString("path_file"));
		pathFileItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a dialog to show and modify file to remember when the app opens.");
		pathFileItem.setActionCommand("pathFile");
		pathFileItem.setFont(font);
		//webItem.setMnemonic(KeyEvent.VK_I);
		//pathFileItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("path_file_mnemonic")).getKeyCode());
		pathFileItem.addActionListener(this);
		this.add(pathFileItem);		
		
		

/*		if ( !  PeaProperties.getFileType().equals("file")) {

			JMenuItem cloudFileItem = new JMenuItem(languagesBundle.getString("cloud_file"));
			cloudFileItem.getAccessibleContext().
			setAccessibleDescription(
					"Clicking this component opens a dialog to show and modify cloud files to remember when the app opens.");
			cloudFileItem.setActionCommand("cloudFile");
			cloudFileItem.setFont(font);
			//webItem.setMnemonic(KeyEvent.VK_I);
			cloudFileItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("cloud_file_mnemonic")).getKeyCode());
			cloudFileItem.addActionListener(this);
			this.add(cloudFileItem);		
		}*/

		JMenuItem aboutItem = new JMenuItem(languagesBundle.getString("about"));
		aboutItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component shows infos about this program.");
		aboutItem.setActionCommand("about");
		aboutItem.setFont(font);
		aboutItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("update_mnemonic")).getKeyCode());
		aboutItem.addActionListener(this);
		this.add(aboutItem);
//TestLog.o(getClass(), PeaSettings.getKeyFileProperty() + "   " + owner.getName());
		if (PeaSettings.getKeyFileProperty() == true
				&& ! (owner instanceof PswDialogView) ) { // only in lock frame
			JMenuItem keyPropItem = new JMenuItem(languagesBundle.getString("show_key_prop"));
			keyPropItem.getAccessibleContext().
		    setAccessibleDescription(
		    	    "Clicking this component shows the key of the program as hexadecimal value after a password entry.");
			keyPropItem.setActionCommand("keyProp");
			keyPropItem.setFont(font);
			keyPropItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("update_mnemonic")).getKeyCode());
			keyPropItem.addActionListener(this);
			this.add(keyPropItem);
		}
		
		JMenuItem checkUpdateItem = new JMenuItem(languagesBundle.getString("check_update"));
		checkUpdateItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component shows infos about available updates of the program.");
		checkUpdateItem.setActionCommand("checkUpdate");
		checkUpdateItem.setFont(font);
		checkUpdateItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("about_mnemonic")).getKeyCode());
		checkUpdateItem.addActionListener(this);
		this.add(checkUpdateItem);
	}

	
	@Override
	public void actionPerformed(ActionEvent ae) {
		String command = ae.getActionCommand();
		
		String fileType = PeaProperties.getFileType();
		// get the parent window: PswDialog or ContentUtilities

		//TestLog.o(getClass(), "command: " + command);
		if (command.equals("instruction")){
			String message = null;
			if (fileType.equals("file")){
				message = PeaProperties.getVmBridge().translate("file_instruction");
			} else if (fileType.equals("image")) {
				message = PeaProperties.getVmBridge().translate("image_instruction");
			} else if (fileType.equals("text")) {
				message = PeaProperties.getVmBridge().translate("text_instruction");
			} else if (fileType.equals("text file")) {
				message = PeaProperties.getVmBridge().translate("text_instruction");
			} else if (fileType.equals("calendar")) {
				message = PeaProperties.getVmBridge().translate("calendar_instruction");
			} else {
				message = "invalid PEA type: " + fileType;
			}
			JOptionPane.showMessageDialog(SwingUtilities.getWindowAncestor(this),//parentWindow,
					message,									
					null, 									   
					JOptionPane.INFORMATION_MESSAGE);

		} else if (command.equals("web")){
			
			PswDialogView.showInfoDialog(SwingUtilities.getWindowAncestor(this));//parentWindow);

		} else if (command.equals("remembered-files")){
			
			PeaDialog.showMessage(this, languagesBundle.getString("edit_remembered_files_text"), null, -1);
			
		} else if (command.equals("pathFile")){

			Point point = SwingUtilities.getWindowAncestor(this).getLocationOnScreen();
			PathFileSetting ps = PathFileSetting.getInstance(SwingUtilities.getWindowAncestor(this), point);
			if (ps.isVisible() == false) {
				ps.setVisible(true);
			}

		} else if (command.equals("checkUpdate")) {
			TestLog.o(getClass(), "check for updates...");
			// create PAD file url: 
			// e.g. https://eck.cologne/peafactory/src/PAD/calendarPAD.xml
			String baseUrl = "https://eck.cologne/peafactory/src/PAD/";
			String peaExtension = null;
			String appType = PeaProperties.getFileType();
			if (appType.equals("calendar")) {
				peaExtension = "calendarPAD.xml";
			} else if (appType.equals("text file")) {
				peaExtension = "notebookPAD.xml";
			} else if (appType.equals("file")) {
				peaExtension = "filePAD.xml";
			} else if (appType.equals("image")) {
				peaExtension = "imagePAD.xml";
			} else {
				new UnexpectedValueException("String", "appType", "is not valid: " + appType).printDescription();
				PeaDialog.showMessage(this, languagesBundle.getString("unexpected_error"), 
						languagesBundle.getString("error"), 0);
				return;
			}
			String urlString = baseUrl + peaExtension;
			if (WebTools.checkConnection() == false){
				PeaDialog.showMessage(this, languagesBundle.getString("bad_web_connection"), 
						languagesBundle.getString("error"), 0);
				return;
			}
			File file = WebTools.getFileFromURL(urlString);
			if (file == null) {
				PeaDialog.showMessage(this, languagesBundle.getString("download_failed"), 
						languagesBundle.getString("error"), 0);
				return;				
			} else {
				DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory
				        .newInstance();
				DocumentBuilder documentBuilder;
				try {
					documentBuilder = documentBuilderFactory.newDocumentBuilder();
					Document document = documentBuilder.parse(file);
					String availableVersion = document.getElementsByTagName("Program_Version").item(0).getTextContent();
					// get release status: Major Update|Minor Update|New Release|Beta|Alpha|Media Only
					String releaseStatus = document.getElementsByTagName("Program_Release_Status").item(0).getTextContent();
					
					String currentVersion = PeaControl.getDialog().getVersion();
					String currentVersionNumber = currentVersion.replaceAll("[^0-9.]", "");
					TestLog.o(getClass(), "current version: " + currentVersion
							+ ", available version: " + availableVersion + ", release status: " + releaseStatus);
					double av = Double.parseDouble(availableVersion);
					double cv = Double.parseDouble(currentVersionNumber);
					if (cv < av) {
						// always show new version
					} else if (cv == av) {
						// check release status:
						if (currentVersion.contains("alpha")) {
							if ( releaseStatus.equals("Alpha")) {
								PeaDialog.showMessage(this, languagesBundle.getString("no_update_found"));
								return;
							} else {
								// show new version
							}
						} else if (currentVersion.contains("beta")) {
							if ( releaseStatus.equals("Beta") || releaseStatus.equals("Alpha")) {
								PeaDialog.showMessage(this, languagesBundle.getString("no_update_found"));
								return;
							} else {
								// show new version
							}		
						}						
					} else {
						PeaDialog.showMessage(this, languagesBundle.getString("no_update_found"));
						return;
					}
					// TODO clickable link and/or download option
					// TODO date of new version, changes...
					PeaDialog.showMultiLineMessage(this, 
							languagesBundle.getString("update_found") + "\n      " 
									+ languagesBundle.getString("version") + " " + av
									+ " (" + releaseStatus + ")"
							+ "\n\n" + languagesBundle.getString("web") + ": \n\n" 
							//+  "<br>" + languagesBundle.getString("web") + "<a href="//\"\">"
							//+ "\"http://eck.cologne/peafactory/en/" + PeaControl.getDialog().getWebsite() + "\">"
							+ "http://eck.cologne/peafactory/en/" + PeaControl.getDialog().getWebsite()
							//+ "</a></HTML>"
							, null, -1);
				} catch (ParserConfigurationException e) {
					TestLog.e(getClass(), new ExceptionHelper(e).getInfos());
					PeaDialog.showMessage(this, languagesBundle.getString("unexpected_error"), 
							languagesBundle.getString("error"), 0);
				} catch (Exception e) {
					TestLog.e(getClass(), new ExceptionHelper(e).getInfos());
					PeaDialog.showMessage(this, languagesBundle.getString("unexpected_error"), 
							languagesBundle.getString("error"), 0);
				}
			}
			
		} else if (command.equals("about")) {
			String peaName = null;
			if (PeaProperties.getFileType().equals("image")){
				peaName = "Image Lock PEA";
			} else if (PeaProperties.getFileType().equals("file")){
				peaName = "File Lock PEA";
			} else if (PeaProperties.getFileType().equals("text file")){
				peaName = "Notebook PEA";
			} else if (PeaProperties.getFileType().equals("calendar")) {
				peaName = "Calendar Lock PEA";
			}
			PswDialogView.showAboutDialog(peaName, null, SwingUtilities.getWindowAncestor(this));
/*			if (PeaControl.getDialog().getLockFrame() != null) {
				// set ContentUtilities as parent
				PswDialogView.showAboutDialog("Notebook PEA", null, PeaControl.getDialog().getLockFrame());
			} else { // in PswDialog: set PswDialog as parent
				PswDialogView.showAboutDialog("Notebook PEA", null, PswDialogView.getView());
			}*/
		} else if (command.equals("keyProp")) {
			// password dialog
			char[] psw = PeaDialog.showPasswordDialog(this, null, 
					PeaProperties.getVmBridge().translate("key_file_property"), true);
			if (psw == null || psw.length == 0) {
				psw = "no password".toCharArray();
			}
			byte[] pswInputBytes = Converter.chars2bytes(psw); 
			// derive key and compare
			byte[] pswHash = HashStuff.generateHKDFBytes(pswInputBytes, 
					Attachments.getProgramRandomBytes(), // if no key file property: this is a fixed string
					64); // size of pswHash: 64 bytes
			byte[] keyMaterial = KeyDerivation.getKdf().deriveKey(pswHash);
			if (keyMaterial == null) {
				PeaDialog.showMessage(this.owner, PeaProperties.getVmBridge().translate("unexpected_error"));
				TestLog.e(getClass(), "Key derivation failed");
				return;
			} 
			boolean fits = Comparator.compare(CipherStuff.getSessionKeyCrypt().getKey(), 
					keyMaterial);
			if (fits == false) {
				PeaDialog.showMessage(this.owner, PeaProperties.getVmBridge().translate("password_failed"));
				TestLog.e(getClass(), "Wrong password");
				return;
			}
			// get value and convert in hex
			String secret = Converter.bytes2hex(Attachments.getProgramRandomBytes());
			// TODO QR code (zxing.jar)
			JPanel keyPanel = new JPanel();
			keyPanel.setLayout(new BoxLayout(keyPanel, BoxLayout.PAGE_AXIS));
			JTextField textField = new JTextField();//Attachments.getProgramRandomBytesSize() * 2);
			//textField.setBorder(null);
			//textArea.setBackground(PeaProperties.getPlainPeaColor());
			textField.setText(secret);
			textField.setEditable(false);
			keyPanel.add(textField);
			JButton keyButton = new JButton(PeaProperties.getVmBridge().translate("copy"));
			keyButton.addActionListener(this);
			keyButton.setActionCommand("copyKey");
			keyPanel.add(keyButton);
			
			JOptionPane.showMessageDialog(this, 
					keyPanel, 
					PeaProperties.getVmBridge().translate("key_file_property"), 1, 
					new ImageIcon(PswDialogView.getImage()));
		} else if (command.equals("copyKey")) {
			String key = ((JTextField)((JPanel)((JButton)ae.getSource()).getParent()).getComponent(0)).getText();
			StringSelection selection = new StringSelection(key);
			Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
			clipboard.setContents(selection, selection);
		} else {
			TestLog.o(getClass(), "Invalid command in help menu");
		}
	}
    public AccessibleContext getAccessibleContext() {
        if (accessibleContext == null) {
            accessibleContext = new AccessibleHelpMenu();
        }
        return accessibleContext;
    }

    private class AccessibleHelpMenu extends AccessibleJMenu {
        //Inherit everything, override nothing.
    }
}
