package cologne.eck.tools;


/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2020  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

import cologne.eck.all_peas.data.PeaProperties;

public class TestLog {
	
	//private static boolean testMode = PeaProperties.isTestMode();
	
	/**
	 * Log a message: In test mode along with '<', class name,
	 * method and line number, in normal mode only the message
	 * 
	 * @param c			the class (X.class)
	 * @param message	the message
	 */
	public static final void o(Class<?> c, String message) {
		
		if (PeaProperties.isTestMode() == true) {// test mode
			System.out.print("< " + c.getSimpleName());
			printOneLine(c.getSimpleName(), message, false);
		} else {
			System.out.println(message);
		}
	}

	/**
	 * Log a message in test mode. 
	 * This will write the class name, method name and line number along
	 * with the message to the standard output if the PEA runs in test mode.
	 * 
	 * @param c			the class (X.class)
	 * @param message	the message
	 */
	public static final void v(Class<?> c, String message) {
		
		if (PeaProperties.isTestMode() == true) {// test mode
			System.out.print(c.getSimpleName());
			printOneLine(c.getSimpleName(), message, false);
		}
	}
	
	/**
	 * Log a message in test mode. 
	 * This will write the class name, method name and line number along
	 * with the message to the standard output if the PEA runs in test mode.
	 * 
	 * @param className	the class name This.getClass().getSimpleName()
	 * @param message	the message
	 */
	public static final void v(String className, String message) {
		
		if (PeaProperties.isTestMode() == true) {// test mode
			System.out.print(className);
			printOneLine(className, message, false);
		}
	}
	
	/**
	 * Log a message in test mode. 
	 * This will write the class name, method name and line number along
	 * with the message to the standard output if the PEA runs in test mode.
	 * 
	 * @param c			the class (X.class)
	 * @param message	the message
	 * @param lineNumber	the number of lines from stack to print
	 * 						- only cologne.eck elements are printed
	 */
	public static final void v(Class<?> c, String message, int lineNumber) {
		
		if (PeaProperties.isTestMode() == true) {// test mode
			printMultiLine(c.getSimpleName(), message, lineNumber, false);
		}
	}
	/**
	 * Log a message in test mode. 
	 * This will write the class name, method name and line number along
	 * with the message to the standard output if the PEA runs in test mode.
	 * 
	 * @param className		the simple class name
	 * @param message		the message
	 * @param lineNumber	the number of lines from stack to print
	 * 						- only cologne.eck elements are printed
	 */
	public static final void v(String className, String message, int lineNumber) {

		if (PeaProperties.isTestMode() == true) {// test mode
			printMultiLine(className, message, lineNumber, false);
		}
	}

	
	/**
	 * Log an error message if program runs in test mode
	 * 
	 * @param c			the class (X.class)
	 * @param message	the message
	 */
	public static final void ve(Class<?> c, String message) {
		
		if (PeaProperties.isTestMode() == true) {
			System.err.print(c.getSimpleName());
			printOneLine(c.getSimpleName(), message, true);
		}
	}
	/**
	 * Log an error message if program runs in test mode
	 * This will write the class name, method name and line number along
	 * with the message to the error output and a number of stack lines.
	 * 
	 * @param className		the class (X.class)
	 * @param message	the message
	 * @param lineNumber	the number of lines from stack to print
	 * 						- only cologne.eck elements are printed
	 */
	public static final void ve(Class<?> c, String message, int lineNumber) {

		if (PeaProperties.isTestMode() == true) {
			printMultiLine(c.getSimpleName(), message, lineNumber, true);
		}
	}
	/**
	 * Log an error message if program runs in test mode
	 * This will write the class name, method name and line number along
	 * with the message to the error output.
	 * 
	 * @param className		the class name this.getClass().getSimpleName()
	 * @param message	the message
	 */
	public static final void ve(String className, String message) {
		
		if (PeaProperties.isTestMode() == true) {// test mode
			System.out.print(className);
			printOneLine(className, message, true);
		}
	}
	
	/**
	 * Log an error message if program runs in test mode
	 * This will write the class name, method name and line number along
	 * with the message to the error output and a number of stack lines.
	 * 
	 * @param className		the class name this.getClass().getSimpleName()
	 * @param message	the message
	 * @param lineNumber	the number of lines from stack to print
	 * 						- only cologne.eck elements are printed
	 */
	public static final void ve(String className, String message, int lineNumber) {

		if (PeaProperties.isTestMode() == true) {
			printMultiLine(className, message, lineNumber, true);
		}
	}
	
	/**
	 * Log a message (also if not in test mode). 
	 * This will write the class name, method name and line number along
	 * with the message to the error output.
	 * 
	 * @param c			the class MethodHandles.lookup().lookupClass()
	 * @param message	the message
	 */
	public static final void e(Class<?> c, String message) {
		System.err.print(c.getSimpleName());
		printOneLine(c.getSimpleName(), message, true);
	}
	/**
	 * Log a message (also if not in test mode). 
	 * This will write the class name, method name and line number along
	 * with the message to the error output.
	 * 
	 * @param className	the class name
	 * @param message	the message
	 */
	public static final void e(String className, String message) {
		System.err.print(className);
		printOneLine(className, message, true);
	}
	/**
	 * Log a message (also if not in test mode). 
	 * This will write the class name, method name and line number along
	 * with the message to the error output.
	 * 
	 * @param c			the class MethodHandles.lookup().lookupClass()
	 * @param message	the message
	 * @param lineNumber	the number of line to print
	 */
	public static final void e(Class<?> c, String message, int lineNumber) {
		System.err.print(c.getSimpleName());
		printMultiLine(c.getSimpleName(), message, lineNumber, true);
	}
	/**
	 * Log a message (also if not in test mode). 
	 * This will write the class name, method name and line number along
	 * with the message to the error output.
	 * 
	 * @param className	the class name
	 * @param message	the message
	 * @param lineNumber	the number of line to print
	 */
	public static final void e(String className, String message, int lineNumber) {
		System.err.print(className);
		printMultiLine(className, message, lineNumber, true);
	}
	
	//==================Helper functions==================================
	
	private static final void printOneLine(String className, String message, boolean err) {
		StackTraceElement[] ste = Thread.currentThread().getStackTrace();
		if (err == true) {		
			System.err.print("." + ste[3].getMethodName());
			System.err.print(" " + ste[3].getLineNumber() + ": ");
			System.err.println(message);
		} else {
			System.out.print("." + ste[3].getMethodName());
			System.out.print(" " + ste[3].getLineNumber() + ": ");
			System.out.println(message);
		}
	}
	private static final void printMultiLine(String className, String message, int lineNumber, boolean err) {
		StackTraceElement[] ste = Thread.currentThread().getStackTrace();
		if (err == true) {
			System.err.print(className);
			System.err.print("." + ste[3].getMethodName());
			System.err.print(" " + ste[3].getLineNumber() + ": ");
			System.err.println(message);

			int stackLen = ste.length;
			int lineCounter = 0;
			for (int i = 3; i < stackLen; i++) {
				if (i == 0) { // always add first line
					System.err.println("\tat " + ste[i]);
				}
				if ( ste[i].getClassName().startsWith("cologne.eck")) {
					System.err.println("\tat " + ste[i]);
					lineCounter++;
				}
				if (lineCounter > lineNumber) {
					break;
				}
			}
		} else {
			System.out.print(className);
			System.out.print("." + ste[3].getMethodName());
			System.out.print(" " + ste[3].getLineNumber() + ": ");
			System.out.println(message);

			int stackLen = ste.length;
			int lineCounter = 0;
			for (int i = 3; i < stackLen; i++) {
				if (i == 0) { // always add first line
					System.out.println("\tat " + ste[i]);
				}
				if ( ste[i].getClassName().startsWith("cologne.eck")) {
					System.out.println("\tat " + ste[i]);
					lineCounter++;
				}
				if (lineCounter > lineNumber) {
					break;
				}
			}
		}
	}	
}
